'use strict';
/* globals lInfo, lWarn, registerState, escape */

/**
 @file md-sr.js

 @brief This file contain implementation of scene responder setup methods.

 @author: Gregor

 @date: 17.06.2013
*/


/**
    Global variable defines if SR should force the action (true) or not (false)
*/
var MD_SR_FORCE = true;
/** Global variable defines if SR is visible in the SR UI:
    - empty or "system-addon-scene-responder", visible
    - otherwise not visible */
var MD_UI_SCOPE = 'motion-detector'; // the name must be exact as app name
// var MD_UI_SCOPE = "";

var APP_ID = 'motion-detector';

/**
 @brief Creates motion state name on passed room ID

 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomMotionStateName (nRoomID) {
    return 'zone.' + nRoomID + '.motion';
}

/**
 @brief Creates presence state name on passed room ID

 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomPresenceStateName (nRoomID) {
    return 'zone.' + nRoomID + '.presence';
}


/**
 @brief Creates light state name on passed room ID

 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomLightStateName (nRoomID) {
    return 'zone.' + nRoomID + '.light';
}


/**
 @brief Creates manual switch status state name on passed room ID

 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomManualSwitchStateName (nRoomID) {
    return 'manual_switch_state_' + nRoomID;
}


/**
 @brief Creates fade off status state name on passed room ID


 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomFadeOffStateName (nRoomID) {
    return 'fade_off_state_' + nRoomID;
}


/**
 @brief Creates final off status state name on passed room ID

 @param nRoomID - room ID

 @return State name
*/
function MD_buildRoomFinalOffStateName (nRoomID) {
    return 'final_off_state_' + nRoomID;
}


/**
 @brief Configure the FINAL DELAY OFF SR for specific room


 @param nRoomID - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nDelay  - delay in seconds

 @return FINAL DELAY OFF SR is created, updated or deleted
*/
function MD_setupFinalOffDelaySR (nRoomID, nDelay) {
    try {
        lInfo.logln('MD_setupFinalOffDelaySR for roomID: ' + nRoomID);

        if (nDelay > 0) {
            var systemStatesFinalOffDelay = [];

            var roomManualSwitchStateName = MD_buildRoomManualSwitchStateName(nRoomID);
            // var roomLightStateName = MD_buildRoomLightStateName(nRoomID);        // not used?
            var roomFadeOffStateName = MD_buildRoomFadeOffStateName(nRoomID);
            var roomFinalOffStateName = MD_buildRoomFinalOffStateName(nRoomID);

            // systemStatesAutoOff.push({ 'name': 'presence', 'value': 'present'}); // present, 1

            // For delay SR has trigger manual switch ON
            var triggersFinalOffDelay = [
                {
                    'type': 'addon-state-change',
                    'name': roomFadeOffStateName,
                    'state': 'active', // '1' // 'active'
                    'addonId': APP_ID
                }
            ];

            var actionsFinalOffDelay = [
                {
                    'type': 'change-addon-state',
                    'statename': roomFinalOffStateName,
                    'state': 'active',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                }
            ];

            // lights can be turned OFF only if NO manual switch
            var addonStatesFinalOffDelay = [
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            ];

            // Create a delay SR for the final off action
            var srFinalOffDelay = {
                'name': 'md_' + nRoomID + '_final_off_delay',
                'id': 'md_' + nRoomID + '_final_off_delay',
                'delay': nDelay,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'singularTriggered': true,
                'triggers': triggersFinalOffDelay,
                'actions': actionsFinalOffDelay,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': null,
                    'systemState': systemStatesFinalOffDelay,
                    'addonStates': {
                        'motion-detector': addonStatesFinalOffDelay
                    }
                }
            };

            var evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srFinalOffDelay))
            });
            evt.raise();
            lInfo.logln('Created SR Delay md_' + nRoomID + '_final_off_delay');
        }
    } catch (e) {
        lWarn.logln('Error by saving MD_setupFinalOffDelaySR: ' + e);
        for (var i in e) {
            if (e.hasOwnProperty(i)) {
                lWarn.logln(i + ': ' + e[i]);
            }
        }
    }
}


/**

 @brief Configure the ALARM SR for specific room

 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bAlarmMotion        - true if also alarm functionality is supported, otherwise false

 @return 2 ALARM SR (set alarm, undo alarm) are created, updated or deleted
*/
function MD_setupAlarmSR (nRoomID, nRoomModeID, bAlarmMotion, nDetectorType) {
    try {
        lInfo.logln('MD_setupAlarmSR for roomID: ' + nRoomID + ', modeID:' + nRoomModeID);

        var roomStateName = MD_buildRoomMotionStateName(nRoomID);
        if (nDetectorType == 1) {
            roomStateName = MD_buildRoomPresenceStateName(nRoomID);
        }

        if ((bAlarmMotion) && (nRoomModeID != 4)/* && (nRoomModeID != 5)*/) {
            var systemStatesAlarm = [];
            var systemStatesUndoAlarm = [];
            var triggersAlarm = [];
            var triggersUndoAlarm = [];

            systemStatesAlarm.push({ 'name': 'presence', 'value': 'absent' }); // absent, 2
            systemStatesUndoAlarm.push({ 'name': 'presence', 'value': 'absent' }); // absent, 2

            systemStatesAlarm.push(
                {
                    'name': 'alarm',
                    'value': '2' // inactive // false
                }
            );

            // var roomLightStateName = MD_buildRoomLightStateName(nRoomID);        // not used?

            triggersAlarm = [
                {
                    'type': 'state-change',
                    'state': 'active',
                    'name': roomStateName
                }
            ];

            triggersUndoAlarm = [
                {
                    'type': 'state-change',
                    'state': 'inactive',
                    'name': roomStateName
                }
            ];

            // Alarm mode is ON, create new SR to signal the alarm
            lInfo.logln('alarm mode is ON');
            // Create alarm SR
            var srAlarm = {
                'name': 'md_' + nRoomID + '_alarm',
                'id': 'md_' + nRoomID + '_alarm',
                'delay': 0,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'triggers': triggersAlarm,
                'actions': [ {
                    'type': 'zone-scene',
                    'zone': 0, // broadcast
                    'group': 0,
                    'scene': 74, // alarm signal
                    'delay': 0,
                    'category': 'manual'
                } ],
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': null,
                    'systemState': systemStatesAlarm,
                    'addonStates': null
                }
            };
            var evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srAlarm))
            });
            evt.raise();
            lInfo.logln('Created SR Alarm md_' + nRoomID + '_alarm');

            // Create undo alarm SR
            var srUndoAlarm = {
                'name': 'md_' + nRoomID + '_undo_alarm',
                'id': 'md_' + nRoomID + '_undo_alarm',
                'delay': 0,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'triggers': triggersUndoAlarm,
                'actions': [ {
                    'type': 'undo-zone-scene',
                    'zone': 0, // broadcast
                    'group': 0,
                    'scene': 74, // alarm signal
                    'delay': 0,
                    'category': 'manual'
                } ],
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': null,
                    'systemState': systemStatesUndoAlarm,
                    'addonStates': null
                }
            };
            var evtUndo = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srUndoAlarm))
            });
            evtUndo.raise();
            lInfo.logln('Created SR Undo Alarm md_' + nRoomID + '_undo_alarm');
        } else { // if ((nRoomModeID != 4) && (nRoomModeID != 5))
            // Delete created SR
            MD_removeAlarmSceneResponder(nRoomID);
        }
    } catch (e) {
        lWarn.logln('Error by saving in MD_setupAlarmSR: ' + e);
        for (var i in e) {
            if (e.hasOwnProperty(i)) {
                lWarn.logln(i + ': ' + e[i]);
            }
        }
    }
}


/**
 @brief Configure the AUTO OFF SR for specific room

 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bAutoOff            - true if manually switched lights should be turned off, otherwise false
 @param nAutoOffDelay       - delay in seconds to turn off the manually switched lights
 @param nAdditionalOffDelay - delay in seconds when lights should be really switched off
                              after there was no motion in the room

 @return AUTO OFF SR is created, updated or deleted
*/
function MD_setupAutoOffSR (nRoomID, nRoomModeID, bAutoOff, nAutoOffDelay, nAdditionalOffDelay, nDetectorType) {
    try {
        lInfo.logln('MD_setupAutoOffSR for roomID: ' + nRoomID + ', modeID:' + nRoomModeID);

        if (bAutoOff && nAutoOffDelay > 0) {
            var systemStatesAutoOff = [];
            var addonStatesAutoOff = [];

            var roomManualSwitchStateName = MD_buildRoomManualSwitchStateName(nRoomID);
            var roomLightStateName = MD_buildRoomLightStateName(nRoomID);
            var roomStateName = MD_buildRoomMotionStateName(nRoomID);
            if (nDetectorType == 1) {
                roomStateName = MD_buildRoomPresenceStateName(nRoomID);
            }
            var roomFadeOffStateName = MD_buildRoomFadeOffStateName(nRoomID);
            var roomFinalOffStateName = MD_buildRoomFinalOffStateName(nRoomID);

            // systemStatesAutoOff.push({ 'name': 'presence', 'value': 'present'}); // present, 1

            var triggersAutoOff = [];
            var actionsAutoOff = [];

            if (nRoomModeID == 5) {
                // For switch-off mode OFF SR has trigger manual switch ON
/*                triggersAutoOff.push(
                    {
                        'type': 'addon-state-change',
                        'name': roomManualSwitchStateName,
                        'state': 'active', // '1' // 'active'
                        'addonId': APP_ID
                    }
                );
*/
                actionsAutoOff.push(
                    {
                        'type': 'change-addon-state',
                        'statename': roomManualSwitchStateName,
                        'state': 'inactive',
                        'delay': 0,
                        'addonId': APP_ID,
                        'category': 'manual'
                    }
                );

                /*actionsAutoOff.push(
                    {
                        'delay': 0,
                        'force': MD_SR_FORCE,
                        'group': 1,
                        'scene': 0,
                        'type': 'zone-scene',
                        'zone': nRoomID,
                        'category': 'manual'
                    }
                );*/

                actionsAutoOff.push(
                    {
                        'type': 'change-addon-state',
                        'statename': roomFadeOffStateName,
                        'state': 'active',
                        'delay': 0,
                        'addonId': APP_ID,
                        'category': 'manual'
                    }
                );

                actionsAutoOff.push(
                    {
                        'type': 'change-addon-state',
                        'statename': roomFinalOffStateName,
                        'state': 'inactive',
                        'delay': 0,
                        'addonId': APP_ID,
                        'category': 'manual'
                    }
                );

                actionsAutoOff.push(
                    {
                        'delay': 0,
                        // 'force': MD_SR_FORCE,
                        'group': 1,
                        'scene': 40, // fade off
                        'type': 'zone-scene',
                        'zone': nRoomID,
                        'category': 'manual'
                    }
                );
            } else {
                if (nRoomModeID != 4) {
                    // lights can be turned OFF only if NO manual switch
                    addonStatesAutoOff.push(
                        {
                            'name': roomManualSwitchStateName,
                            'value': false // '2' // 'inactive'
                        }
                    );
                }

                actionsAutoOff.push(
                    {
                        'delay': 0,
                        'force': MD_SR_FORCE,
                        'group': 1,
                        'scene': 0,
                        'type': 'zone-scene',
                        'zone': nRoomID,
                        'category': 'manual'
                    }
                );
            }

            triggersAutoOff.push(
                { // preset 1
                    'type': 'zone-scene',
                    'zone': nRoomID,
                    'group': 1,
                    'scene': 5
                }
            );

            systemStatesAutoOff.push(
                {
                    'name': roomLightStateName,
                    'value': '1' // 'active'
                }
            );

            // lights can be turned OFF only if NO motion
            systemStatesAutoOff.push(
                {
                    'name': roomStateName,
                    'value': '2' // 'inactive'
                }
            );

            // Create switch off SR with AUTO OFF action
            var srAutoOff = {
                'name': 'md_' + nRoomID + '_auto_off',
                'id': 'md_' + nRoomID + '_auto_off',
                'delay': nAutoOffDelay + nAdditionalOffDelay,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'singularTriggered': true,
                'triggers': triggersAutoOff,
                'actions': actionsAutoOff,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': null,
                    'systemState': systemStatesAutoOff,
                    'addonStates': {
                        'motion-detector': addonStatesAutoOff
                    }
                }
            };
            var evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srAutoOff))
            });
            evt.raise();
            lInfo.logln('Created SR Auto Off md_' + nRoomID + '_auto_off');
        } else {
            // Delete created SR
            MD_removeAutoOffSceneResponder(nRoomID);
        }
    } catch (e) {
        lWarn.logln('Error by saving MD_setupAutoOffSR: ' + e);
        for (var i in e) {
            if (e.hasOwnProperty(i)) {
                lWarn.logln(i + ': ' + e[i]);
            }
        }
    }
}


/**
 @brief Configure the 2 SRs to create manual state active/inactive
        for manual turn on/off the light in the specific room


 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,

 @return 2 SRs for are created, updated or deleted
*/
function MD_setupManualSwitchSR (nRoomID, nRoomModeID) {
    try {
        lInfo.logln('MD_setupManualSwitchSR for roomID: ' + nRoomID + ', modeID:' + nRoomModeID);

        var roomManualSwitchStateName = MD_buildRoomManualSwitchStateName(nRoomID);
        // var roomLightStateName = MD_buildRoomLightStateName(nRoomID);        // not used?
        var roomFadeOffStateName = MD_buildRoomFadeOffStateName(nRoomID);

        // --------------------------------------------------------
        // Create SR to set state aka 'manual_switch' to active

        var systemStatesManualOn = [];

        var triggersManualOn = [
            { // preset 2
                'type': 'zone-scene',
                'zone': nRoomID,
                'group': 1,
                'scene': 17
            },
            { // preset 3
                'type': 'zone-scene',
                'zone': nRoomID,
                'group': 1,
                'scene': 18
            },
            { // preset 4
                'type': 'zone-scene',
                'zone': nRoomID,
                'group': 1,
                'scene': 19
            }
        ];

        // Create SR to create/set the state for manual switch ON
        var srManualSwitchON = {
            'name': 'md_' + nRoomID + '_set_manual_switch_on',
            'id': 'md_' + nRoomID + '_set_manual_switch_on',
            'delay': 0,
            'scope': MD_UI_SCOPE,
            'persistentScope': false,
            'triggers': triggersManualOn,
            'actions': [
                {
                    'type': 'change-addon-state',
                    'statename': roomManualSwitchStateName,
                    'state': 'active',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                },
                {
                    'type': 'change-addon-state',
                    'statename': roomFadeOffStateName,
                    'state': 'inactive',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                }
            ],
            'conditions': {
                'enabled': true,
                'weekdays': null,
                'timeframe': null,
                'zoneState': null,
                'systemState': systemStatesManualOn,
                'addonStates': null
            }
        };
        var evt = new Event('system-addon-scene-responder.config', {
            actions: 'save',
            value: escape(JSON.stringify(srManualSwitchON))
        });
        evt.raise();
        lInfo.logln('Created SR Manual Switch ON md_' + nRoomID + '_set_manual_switch_on');

        // --------------------------------------------------------
        // Create SR to set state aka 'manual_switch' to inactive

        var addonStatesManualOff = [];

        // manual switch can be turned OFF only if manual switch
        addonStatesManualOff.push(
            {
                'name': roomManualSwitchStateName,
                'value': true // '1' // 'active'
            }
        );

        var triggersManualOff = [
            { // off called
                'type': 'zone-scene',
                'zone': nRoomID,
                'group': 1,
                'scene': 0
            }
        ];

        // Create SR to create/set the state for manual switch OFF
        var srManualSwitchOFF = {
            'name': 'md_' + nRoomID + '_set_manual_switch_off',
            'id': 'md_' + nRoomID + '_set_manual_switch_off',
            'delay': 0,
            'scope': MD_UI_SCOPE,
            'persistentScope': false,
            'triggers': triggersManualOff,
            'actions': [
                {
                    'type': 'change-addon-state',
                    'statename': roomManualSwitchStateName,
                    'state': 'inactive',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                }
            ],
            'conditions': {
                'enabled': true,
                'weekdays': null,
                'timeframe': null,
                'zoneState': null,
                'systemState': null,
                'addonStates': {
                    'motion-detector': addonStatesManualOff
                }
            }
        };
        evt = new Event('system-addon-scene-responder.config', {
            actions: 'save',
            value: escape(JSON.stringify(srManualSwitchOFF))
        });
        evt.raise();
        lInfo.logln('Created SR Manual Switch ON md_' + nRoomID + '_set_manual_switch_off');

    } catch (e) {
        lWarn.logln('Error by saving MD_setupManualSwitchSR: ' + e);
        for (var i in e) {
            if (e.hasOwnProperty(i)) {
                lWarn.logln(i + ': ' + e[i]);
            }
        }
    }
}


/**
 @brief Configure scene responder for specific room with specific mode and it
        either creates new scene responders, delete existing ones or just update
        existing ones with new parameters

 @param nRoomID              - valid room ID if > 0, all rooms if == 0, invalid room if < 0
 @param nRoomModeID          - contains standard mode, where valid numbers are:
                               1 -> Corridor light,
                               2 -> Basement light,
                               3 -> Stairway light,
                               4 -> Outdoor light,
                               5 -> Switch off light,
 @param bSwitchOffDarkOnly  - true to switch lights in dark only, otherwise false
 @param nTwilightSensorID   - valid sensor id if > 0, otherwise use "sun position"
 @param bAlarmMotion        - true if also alarm functionality is supported, otherwise false
 @param bAutoOff            - true if manually switched lights should be turned off, otherwise false
 @param nAutoOffDelay       - delay in seconds to turn off the manually switched lights
 @param nAdditionalOffDelay - delay in seconds when lights should be really switched off
                              after there was no motion in the room
 @param nDetectorType       - detector types (1=presence, 5=motion)

 @return true if scene responders are created/updated/deleted on input parameters
         and false if error occured
*/
function MD_setupSceneResponder (nRoomID, nRoomModeID, bSwitchOffDarkOnly, nTwilightSensorID,
    bAlarmMotion, bAutoOff, nAutoOffDelay, nAdditionalOffDelay, nDetectorType) {
    try {
        lInfo.logln('MD_setupSceneResponder for roomID: ' + nRoomID + ', modeID:' + nRoomModeID +
            ', darkonly: ' + bSwitchOffDarkOnly +
            ', twilight sensor: ' + nTwilightSensorID +
            ', detector type: ' + nDetectorType);

        if (nRoomModeID < 1 || nRoomModeID > 5) {
            // Invalid mode number
            lInfo.logln('Invalid room mode number: ' + nRoomModeID);
            return false;
        }

        var roomStateName = MD_buildRoomMotionStateName(nRoomID);
        if (nDetectorType == 1) {
            roomStateName = MD_buildRoomPresenceStateName(nRoomID);
        }
        var roomLightStateName = MD_buildRoomLightStateName(nRoomID);
        var roomManualSwitchStateName = MD_buildRoomManualSwitchStateName(nRoomID);
        var roomFadeOffStateName = MD_buildRoomFadeOffStateName(nRoomID);
        var roomFinalOffStateName = MD_buildRoomFinalOffStateName(nRoomID);


        registerState(roomManualSwitchStateName, true);
        registerState(roomFadeOffStateName, true);
        registerState(roomFinalOffStateName, true);

        lInfo.logln('room motion state name is: ' + roomStateName);
        lInfo.logln('room light state name is: ' + roomLightStateName);

        // --------------------------------------------------------------------
        // ON related SR implementation

        var systemStatesTurnOnPresent = [];
        var systemStatesTurnOnAbsent = [];
        var addonStatesTurnOnPresent = [];
        var addonStatesTurnOnAbsent = [];
        var zoneStatesTurnOn = [];
        var actionsTurnOnPresent = [];
        var actionsTurnOnAbsent = [];
        var triggersTurnOn = [];

        systemStatesTurnOnPresent.push({ 'name': 'presence', 'value': 'present' }); // present, 1
        systemStatesTurnOnAbsent.push({ 'name': 'presence', 'value': 'absent' }); // absent, 2

        if (nRoomModeID != 5) {
            // Lights can be turned ON only if light state for this room is inactive
            systemStatesTurnOnPresent.push(
                {
                    'name': roomLightStateName,
                    'value': '2' // 'inactive'
                }
            );
        }

        systemStatesTurnOnAbsent.push(
            {
                'name': roomLightStateName,
                'value': '2' // 'inactive'
            }
        );

        // OR if lights are in fadeing off state


        if (nRoomModeID != 3 && nRoomModeID != 4) {
            // lights can be turned ON only if NO manual switch
            addonStatesTurnOnPresent.push(
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            );

            addonStatesTurnOnAbsent.push(
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            );
        }

        systemStatesTurnOnAbsent.push(
            {
                'name': 'alarm',
                'value': '2' // inactive // false
            }
        );

        if ((nRoomModeID == 1 || nRoomModeID == 2) && !bAlarmMotion) {
            lInfo.logln('call come home: ');

            actionsTurnOnAbsent.push(
                {
                    'type': 'zone-scene',
                    'zone': 0, // broadcast
                    'group': 0,
                    'scene': 71, // coming home, present
                    'delay': 0,
                    'category': 'manual'
                }
            );
        }

        actionsTurnOnPresent.push(
            {
                'delay': 0,
                'group': 1,
                'scene': 5, // Should we use local on - 51 ?
                'type': 'zone-scene',
                'zone': nRoomID,
                'category': 'manual'
            }
        );

        actionsTurnOnPresent.push(
            {
                'type': 'change-addon-state',
                'statename': roomFadeOffStateName,
                'state': 'inactive',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        actionsTurnOnAbsent.push(
            {
                'delay': 0,
                'group': 1,
                'scene': 5, // Should we use local on - 51 ?
                'type': 'zone-scene',
                'zone': nRoomID,
                'category': 'manual'
            }
        );

        actionsTurnOnAbsent.push(
            {
                'type': 'change-addon-state',
                'statename': roomFadeOffStateName,
                'state': 'inactive',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        if (nRoomModeID == 5) {
            zoneStatesTurnOn.push(
                {
                    'zone': nRoomID,
                    'group': 1,
                    'scene': 40 // fading
                }
            );
        }

        triggersTurnOn.push(
            {
                'type': 'state-change',
                'state': 'active',
                'name': roomStateName
            }
        );

        if (bSwitchOffDarkOnly && nRoomModeID != 5) {
            // Switch ON if dark only
            lInfo.logln('Switch ON on dark only');

            if (nTwilightSensorID > '0') {
                // Use twilight sensor
                lInfo.logln('Use twilight sensor');
                systemStatesTurnOnPresent.push({ 'name': 'dev.' + nTwilightSensorID + '.0', 'value': '1' });
                systemStatesTurnOnAbsent.push({ 'name': 'dev.' + nTwilightSensorID + '.0', 'value': '1' });
            } else if (nTwilightSensorID == -1) {
                // Use default "position of the sun" parameters
                lInfo.logln('Use \'sunset to sunrise\'');
                // valid values for daynight state: 1 - bright, 2 - dark
                // OR true - bright, false - dark
                systemStatesTurnOnPresent.push({ 'name': 'daynight', 'value': false });
                systemStatesTurnOnAbsent.push({ 'name': 'daynight', 'value': false });
            } else if (nTwilightSensorID == -2) {
                // Use default "position of the sun" parameters
                lInfo.logln('Use \'sunset-1h to sunset+1h\'');
                // valid values for daynight_indoors state: 1 - bright, 2 - dark
                // OR true - bright, false - dark
                systemStatesTurnOnPresent.push({ 'name': 'daynight_indoors', 'value': false });
                systemStatesTurnOnAbsent.push({ 'name': 'daynight_indoors', 'value': false });
            } else {
                // Use default "position of the sun" parameters
                lInfo.logln('Use \'dusk to dawn\'');
                // valid values for daylight state: 1 - bright, 2 - dark
                // OR true - bright, false - dark
                systemStatesTurnOnPresent.push({ 'name': 'daylight', 'value': false });
                systemStatesTurnOnAbsent.push({ 'name': 'daylight', 'value': false });
            }
        }

        lInfo.logln('Create turn on SR');

        // Create motion depend SR with ON action when present
        var srOnPresent = {
            'name': 'md_' + nRoomID + '_turn_on_present',
            'id': 'md_' + nRoomID + '_turn_on_present',
            'delay': 0,
            'scope': MD_UI_SCOPE,
            'persistentScope': false,
            'triggers': triggersTurnOn,
            'actions': actionsTurnOnPresent,
            'conditions': {
                'enabled': true,
                'weekdays': null,
                'timeframe': null,
                'zoneState': zoneStatesTurnOn,
                'systemState': systemStatesTurnOnPresent,
                'addonStates': {
                    'motion-detector':addonStatesTurnOnPresent
                }
            }
        };
        var evt = new Event('system-addon-scene-responder.config', {
            actions: 'save',
            value: escape(JSON.stringify(srOnPresent))
        });
        evt.raise();

        lInfo.logln('Created SR Turn On present md_' + nRoomID + '_turn_on_present');

        // if (((nRoomModeID == 1) || (nRoomModeID == 2)) && (!bAlarmMotion)) {
        if (!bAlarmMotion) {
            // Create motion depend SR with ON action when absent
            var srOnAbsent = {
                'name': 'md_' + nRoomID + '_turn_on_absent',
                'id': 'md_' + nRoomID + '_turn_on_absent',
                'delay': 0,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'triggers': triggersTurnOn,
                'actions': actionsTurnOnAbsent,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': zoneStatesTurnOn,
                    'systemState': systemStatesTurnOnAbsent,
                    'addonStates': {
                        'motion-detector': addonStatesTurnOnAbsent
                    }
                }
            };
            evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srOnAbsent))
            });
            evt.raise();

            lInfo.logln('Created SR Turn On absent md_' + nRoomID + '_turn_on_absent');
        } else {
            MD_removeTurnOnSceneResponderAbsent(nRoomID);
        }

        // -------------------------------------------------------------------
        // OFF related SR implementation

        // var zone_states_auto_off = [];       // not used?

        var zoneStatesFadeOff = [];
        var triggersFadeOff = [];
        var systemStatesFadeOff = [];
        var addonStatesFadeOff = [];
        var actionsFadeOff = [];

        var zoneStatesFinalOff = [];
        var triggersFinalOff = [];
        var systemStatesFinalOff = [];
        var addonStatesFinalOff = [];
        var actionsFinalOff = [];

        var triggersTurnOff = [];
        var systemStatesTurnOff = [];
        var zoneStatesTurnOff = [];
        var addonStatesTurnOff = [];
        var actionsTurnOff = [];

        if (nRoomModeID != 5 && bAlarmMotion) {
            systemStatesTurnOff.push({ 'name': 'presence', 'value': 'present' }); // present, 1
            systemStatesFadeOff.push({ 'name': 'presence', 'value': 'present' }); // present, 1
        }

        // Switch off only if light was turned ON
        // Lights can be turned OFF only if light state for this room is active
        systemStatesTurnOff.push(
            {
                'name': roomLightStateName,
                'value': '1' // 'active'
            }
        );

        // Lights can be turned OFF only if light state for this room is active
        systemStatesFadeOff.push(
            {
                'name': roomLightStateName,
                'value': '1' // 'active'
            }
        );

        // lights can be turned OFF only if NO motion
        systemStatesFadeOff.push(
            {
                'name': roomStateName,
                'value': '2' // 'inactive'
            }
        );

        if (nRoomModeID != 3 && nRoomModeID != 4) {
            // lights can be turned OFF only if NO manual switch
            addonStatesTurnOff.push(
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            );

            // lights can be turned OFF only if NO manual switch
            addonStatesFadeOff.push(
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            );

            // lights can be turned OFF only if NO manual switch
            addonStatesFinalOff.push(
                {
                    'name': roomManualSwitchStateName,
                    'value': false // '2' // 'inactive'
                }
            );
        }

        zoneStatesFinalOff.push(
            {
                'zone': nRoomID,
                'group': 1,
                'scene': 40
            }
        );

        // lights can be turned OFF only if NO motion
        systemStatesFinalOff.push(
            {
                'name': roomStateName,
                'value': '2' // 'inactive'
            }
        );

        triggersTurnOff.push(
            {
                'type': 'state-change',
                'state': 'inactive',
                'name': roomStateName
            }
        );

        triggersFadeOff.push(
            {
                'type': 'state-change',
                'state': 'inactive',
                'name': roomStateName
            }
        );

        /*if (bAutoOff) {
            // Fade off is triggered on no motion or if preset1
            triggersFadeOff.push(
                {
                    'type': 'zone-scene',
                    'zone': nRoomID,
                    'group': 1,
                    'scene': 5 // preset 1
                }
            );
        }*/

        triggersFinalOff.push(
            {
                'type': 'addon-state-change',
                'name': roomFinalOffStateName,
                'state': 'active', // '1' // 'active'
                'addonId': APP_ID
            }
        );

        if (nRoomModeID == 5) {
            // For switch-off mode OFF SR has trigger preset1
            triggersTurnOff.push(
                {
                    'type': 'addon-state-change',
                    'name': roomManualSwitchStateName,
                    'state': 'active', // '1' // 'active'
                    'addonId': APP_ID
                }
            );

            actionsTurnOff.push(
                {
                    'type': 'change-addon-state',
                    'statename': roomManualSwitchStateName,
                    'state': 'inactive',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                }
            );

            // triggersFadeOff.push(
            //     {
            //         'type': 'addon-state-change',
            //         'name': roomManualSwitchStateName,
            //         'state': 'active', // '1' // 'active'
            //         'addonId': APP_ID
            //     }
            // );

            actionsFadeOff.push(
                {
                    'type': 'change-addon-state',
                    'statename': roomManualSwitchStateName,
                    'state': 'inactive',
                    'delay': 0,
                    'addonId': APP_ID,
                    'category': 'manual'
                }
            );

            /*triggersFinalOff.push(
                                {
                                     'type': 'addon-state-change',
                                     'name': roomManualSwitchStateName,
                                     'state': 'active', // '1' // 'active'
                                     'addonId': APP_ID
                                }
                            );*/
        }

        actionsFadeOff.push(
            {
                'type': 'change-addon-state',
                'statename': roomFadeOffStateName,
                'state': 'active',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        actionsFadeOff.push(
            {
                'type': 'change-addon-state',
                'statename': roomFinalOffStateName,
                'state': 'inactive',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        actionsFadeOff.push(
            {
                'delay': 0,
                'group': 1,
                'scene': 40, // fade off
                'type': 'zone-scene',
                'zone': nRoomID,
                'category': 'manual'
            }
        );

        actionsFinalOff.push(
            {
                'type': 'change-addon-state',
                'statename': roomFadeOffStateName,
                'state': 'inactive',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        actionsFinalOff.push(
            {
                'type': 'change-addon-state',
                'statename': roomFinalOffStateName,
                'state': 'inactive',
                'delay': 0,
                'addonId': APP_ID,
                'category': 'manual'
            }
        );

        actionsFinalOff.push(
            {
                'delay': 0,
                // 'force': MD_SR_FORCE, // do not force
                'group': 1,
                'scene': 0, // off
                'type': 'zone-scene',
                'zone': nRoomID,
                'category': 'manual'
            }
        );

        actionsTurnOff.push(
            {
                'delay': 0,
                'force': MD_SR_FORCE,
                'group': 1,
                'scene': 0, // should we use 50 - local off?
                'type': 'zone-scene',
                'zone': nRoomID,
                'category': 'manual'
            }
        );

        var FADE_OFF_DURATION = 60; /* duration of the fadeoff in seconds */

        if (nRoomModeID != 4) {
            MD_removeTurnOffSceneResponder(nRoomID);

            var nFadeOffDelay = nAdditionalOffDelay;

            /* First index is and mode which can then have four different delays:
               - normal delay
               - darknes only delay
               - with alarm delay
               - darknes + alarm */
            /*
                                                // normal, darkness, alarm, dark+alarm
                var nFadeOffDelaysInSecForMode = [ [180, 300, 300, 300] // corridor
                                                 , [3600, 0, 3600, 0]   // basement
                                                 , [180, 0, 0, 0]       // stairway
                                                 , [3600, 3600, 0, 0]   // outdoor
                                                 , [300, 0, 0, 0]       // auto switch-off
                                                 ];

                if ((bSwitchOffDarkOnly) && (bAlarmMotion)) {
                    nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][3];
                }
                else if (bAlarmMotion) {
                    nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][2];
                }
                else if (bSwitchOffDarkOnly) {
                    nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][1];
                }
                else {
                    nFadeOffDelay = nFadeOffDelaysInSecForMode[nRoomModeID - 1][0];
                }
            */

            // This modes uses fade off to turn the light off

            // Create motion depend fadeoff SR
            var srFadeOff = {
                'name': 'md_' + nRoomID + '_fade_off',
                'id': 'md_' + nRoomID + '_fade_off',
                'delay': nFadeOffDelay,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'singularTriggered': true,
                'triggers': triggersFadeOff,
                'actions': actionsFadeOff,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': zoneStatesFadeOff,
                    'systemState': systemStatesFadeOff,
                    'addonStates': {
                        'motion-detector': addonStatesFadeOff
                    }
                }
            };
            evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srFadeOff))
            });
            evt.raise();
            lInfo.logln('Created SR Fade Off md_' + nRoomID + '_fade_off');

            // another entry is needed for the following special case:
            // meanwhile light is ON user calls "leave home" and there are lamps with "don't care for leave home" setting ON
            // result: zone.N.light state if inactive, while light is actually still on - so it needs to be faded off
            systemStatesFadeOff = systemStatesFadeOff
                .filter(function (condition) {
                    return condition.name !== 'presence' && condition.name !== roomLightStateName;
                });
            systemStatesFadeOff.push({ 'name': 'presence', 'value': 'absent' });
            srFadeOff = {
                'name': 'md_' + nRoomID + '_fade_off_absent',
                'id': 'md_' + nRoomID + '_fade_off_absent',
                'delay': nFadeOffDelay,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'singularTriggered': true,
                'triggers': triggersFadeOff,
                'actions': actionsFadeOff,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': zoneStatesFadeOff,
                    'systemState': systemStatesFadeOff,
                    'addonStates': {
                        'motion-detector': addonStatesFadeOff
                    }
                }
            };
            evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srFadeOff))
            });
            evt.raise();
            lInfo.logln('Created SR Fade Off md_' + nRoomID + '_fade_off_absent for absent state + "don\'t care for leave home" lamps');

            // Create motion depend final off SR
            var srFinalOff = {
                'name': 'md_' + nRoomID + '_final_off',
                'id': 'md_' + nRoomID + '_final_off',
                'delay': 0,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'triggers': triggersFinalOff,
                'actions': actionsFinalOff,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': zoneStatesFinalOff,
                    'systemState': systemStatesFinalOff,
                    'addonStates': {
                        'motion-detector':addonStatesFinalOff
                    }
                }
            };
            var evt_final = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srFinalOff))
            });
            evt_final.raise();
            lInfo.logln('Created SR Final Off md_' + nRoomID + '_final_off');

            MD_setupFinalOffDelaySR(nRoomID, FADE_OFF_DURATION /* Wait for the fade-off to end */);
        } else {
            MD_removeFadeOffSceneResponder(nRoomID);

            MD_removeFinalOffSceneResponder(nRoomID);

            // Create motion depend turn off SR
            var srTurnOff = {
                'name': 'md_' + nRoomID + '_turn_off',
                'id': 'md_' + nRoomID + '_turn_off',
                'delay': nAdditionalOffDelay,
                'scope': MD_UI_SCOPE,
                'persistentScope': false,
                'triggers': triggersTurnOff,
                'actions': actionsTurnOff,
                'conditions': {
                    'enabled': true,
                    'weekdays': null,
                    'timeframe': null,
                    'zoneState': zoneStatesTurnOff,
                    'systemState': systemStatesTurnOff,
                    'addonStates': {
                        'motion-detector': addonStatesTurnOff
                    }
                }
            };
            evt = new Event('system-addon-scene-responder.config', {
                actions: 'save',
                value: escape(JSON.stringify(srTurnOff))
            });
            evt.raise();
            lInfo.logln('Created SR Turn Off md_' + nRoomID + '_turn_off');
        }

        // ---------------------------------------------------------------------
        // Create other SR

        MD_setupAlarmSR(nRoomID, nRoomModeID, bAlarmMotion, nDetectorType);

        // Auto off delay must be bigger than fade off duration - why?
        if (nRoomModeID != 4) {
            nAutoOffDelay = nAutoOffDelay; // + FADE_OFF_DURATION;
        }

        MD_setupAutoOffSR(nRoomID, nRoomModeID, bAutoOff, nAutoOffDelay, 0, nDetectorType); // nAdditionalOffDelay);

        MD_setupManualSwitchSR(nRoomID, nRoomModeID);

    } catch (e) {
        lWarn.logln('Error by saving MD_setupSceneResponder: ' + e);
        for (var i in e) {
            if (e.hasOwnProperty(i)) {
                lWarn.logln(i + ': ' + e[i]);
            }
        }
    }

    return true;
}


/**
 @brief Delete scene responder handling ALARM mode for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling ALARM mode is deleted
*/
function MD_removeAlarmSceneResponder (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_alarm']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();

    sIDString = escape(JSON.stringify([sSRID + '_undo_alarm']));
    var evtUndo = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evtUndo.raise();
}


/**
 @brief Delete scene responder handling AUTO OFF mode for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling AUTO OFF mode is deleted

*/
function MD_removeAutoOffSceneResponder (nRoomID) {
    var sSRID = 'md_' + nRoomID;

    var sIDString = escape(JSON.stringify([sSRID + '_auto_off']));

    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();
}


/**
 @brief Delete scene responder handling FINAL OFF for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling FINAL OFF is deleted
*/
function MD_removeFinalOffSceneResponder (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_final_off']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();

    sIDString = escape(JSON.stringify([sSRID + '_final_off_delay']));
    var evt_delay = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt_delay.raise();
}

/**
 @brief Delete scene responder handling FADE OFF for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling FADE OFF is deleted
*/
function MD_removeFadeOffSceneResponder (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_fade_off']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();

    var sIDString = escape(JSON.stringify([sSRID + '_fade_off_absent']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();
}


/**
 @brief Delete scene responder handling TURN OFF for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling TURN OFF is deleted
*/
function MD_removeTurnOffSceneResponder (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_turn_off']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();
}


/**
 @brief Delete scene responder handling TURN ON for specific room. This is done
        when mode OR its parameters are changed

 @param nRoomID - room ID

 @return Scene responder handling TURN ON is deleted
*/
function MD_removeTurnOnSceneResponderAbsent (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_absent']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();
}


/**
 @brief Delete all scene responders for specific room. This is done
        when "advanced" mode is selected or there are none MDs in the room

 @param roomID - room ID

 @return All scene responders for this room are deleted
*/
function MD_removeAllSceneRespondersForRoom (nRoomID) {
    var sSRID = 'md_' + nRoomID;
    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_present'
                                         , sSRID + '_turn_on_absent'
                                         , sSRID + '_fade_off'
                                         , sSRID + '_fade_off_absent'
                                         , sSRID + '_final_off'
                                         , sSRID + '_final_off_delay'
                                         , sSRID + '_auto_off'
                                         , sSRID + '_turn_off'
                                         , sSRID + '_alarm'
                                         , sSRID + '_undo_alarm'
                                         , sSRID + '_set_manual_switch_on'
                                         , sSRID + '_set_manual_switch_off']));
    var evt = new Event('system-addon-scene-responder.config', {
        'actions': 'delete',
        'id': sIDString
    });
    evt.raise();
}

/**
 @brief Enables/Disables all scene responders for specific room. This is done
        using button Activate/Deactivate on grid or using checkbox "Active" in grid

 @param nRoomID - room ID
 @param bActivate - active the SRs - true, false to disable them

 @return All scene responders for this room are enabled/disabled
*/
function MD_setAllSceneRespondersForRoomState (nRoomID, bActivate) {
    var sSRID = 'md_' + nRoomID;

    var sAction = '';
    if (bActivate) {
        sAction = 'enable';
    } else {
        sAction = 'disable';
    }

    var sIDString = escape(JSON.stringify([sSRID + '_turn_on_present'
                                         , sSRID + '_turn_on_absent'
                                         , sSRID + '_fade_off'
                                         , sSRID + '_fade_off_absent'
                                         , sSRID + '_final_off'
                                         , sSRID + '_final_off_delay'
                                         , sSRID + '_auto_off'
                                         , sSRID + '_turn_off'
                                         , sSRID + '_alarm'
                                         , sSRID + '_undo_alarm'
                                         , sSRID + '_set_manual_switch_on'
                                         , sSRID + '_set_manual_switch_off']));

    var evt = new Event('system-addon-scene-responder.config', {
        'actions': sAction,
        'id': sIDString
    });
    evt.raise();
}
